@@ -45,6 +45,8 @@ namespace giac {
   typedef double long_double;
 #endif
   typedef std::complex<long_double> complex_long_double;
+  double complex_abs(const complex_double & c);
+  double complex_long_abs(const complex_long_double & c);
 
   // make a matrix with free rows 
   // (i.e. it is possible to modify the answer in place)
@@ -305,7 +307,14 @@ namespace giac {
   // finish full row reduction to echelon form if N is upper triangular
   // this is done from lmax-1 to l
   void smallmodrref_upper(std::vector< std::vector<int> > & N,int l,int lmax,int c,int cmax,int modulo);
+  // finish row reduction for matrices with much more columns than rows
+  // version adapted for threads parallelization
+  // assumes that all columns are reduced in parallel, pivots are searched
+  // starting at column 0
+  void in_thread_smallmodrref_upper(std::vector< std::vector<int> > & N,int l,int lpivot,int lmax,int c,int cmax,int modulo,int parallel);
+  void thread_smallmodrref_upper(std::vector< std::vector<int> > & N,int l,int lmax,int c,int cmax,int modulo,int parallel);
   void free_null_lines(std::vector< std::vector<int> > & N,int l,int lmax,int c,int cmax);
+  int smallmodrref_lastpivotcol(const std::vector< std::vector<int> > & K,int lmax);
 
   void smallmodrref(int nthreads,std::vector< std::vector<int> > & N,vecteur & pivots,std::vector<int> & permutation,std::vector<int> & maxrankcols,longlong & idet,int l, int lmax, int c,int cmax,int fullreduction,int dont_swap_below,int modulo,int rref_or_det_or_lu,bool reset,smallmodrref_temp_t * workptr,bool allow_block,int carac);
   void doublerref(matrix_double & N,vecteur & pivots,std::vector<int> & permutation,std::vector<int> & maxrankcols,double & idet,int l, int lmax, int c,int cmax,int fullreduction,int dont_swap_below,int rref_or_det_or_lu,double eps);
@@ -321,6 +330,7 @@ namespace giac {
   bool remove_identity(matrice & res);
   bool remove_identity(std::vector< std::vector<int> > & res,int modulo);
 
+  void mdividebypivot(matrice & a,int lastcol,GIAC_CONTEXT); // in-place div by pivots
   void mdividebypivot(matrice & a,int lastcol=-1); // in-place div by pivots
   // if lastcol==-1, divide last col, if lastcol==-2 do not divide last col
   // if lastcol>=0 stop dividing at lastcol
@@ -329,6 +339,7 @@ namespace giac {
   gen _idn(const gen & e,GIAC_CONTEXT);
   extern const unary_function_ptr * const  at_idn ;
 
+  gen fieldcoeff(const gen &F);
   vecteur vranm(int n,const gen & f,GIAC_CONTEXT); 
   matrice mranm(int n,int m,const gen & f,GIAC_CONTEXT); // random matrix using f
   gen _ranm(const gen & e,GIAC_CONTEXT);
@@ -510,7 +521,7 @@ namespace giac {
   bool hermite(const std_matrix<gen> & Aorig,std_matrix<gen> & U,std_matrix<gen> & A,environment * env,GIAC_CONTEXT);
   gen _ihermite(const gen & g,GIAC_CONTEXT);
   gen _ismith(const gen & g,GIAC_CONTEXT);
-#ifndef NSPIRE
+#if !defined NSPIRE && !defined FXCG
   gen _csv2gen(const gen & g,GIAC_CONTEXT);
   matrice csv2gen(std::istream & i,char sep,char nl,char decsep,char eof,GIAC_CONTEXT);
 #endif
